###
# No Copyright 2008, fv
# No rights reserved.
#
#
###

import supybot.utils as utils
from supybot.commands import *
import supybot.plugins as plugins
import supybot.ircutils as ircutils
import supybot.callbacks as callbacks

import serial #import serial library
import sys, time


class StepperBot(callbacks.Plugin):
	"""translates irc commands like right, left, up, down 
	to artbus stepper comands, which are then sent 
	via serial connection."""
	
	def __init__(self, irc):
		self.__parent = super(StepperBot, self)
		self.__parent.__init__(irc)
		
		# flag to set if script shall run as test-only without serial connection or live with serial connection
		self.TESTF = False 
		
		# sequence of patterns corresponding to direction commands (right, left, down, up)
		# each sequence of 4 numbers is for the 4 pins 
		# (adapt to your needs)
		"""
		right_pattern = ((1, 0, 0 , 0), (0, 1,  0, 0 ), (0, 0, 1 , 0), (0, 0, 0, 1))
		left_pattern = ((0, 0, 0 , 1), (0, 1,  0, 0 ), (0, 0, 1 , 0), (0, 0, 0, 1))
		up_pattern = ((1, 1, 1 , 1), (0, 1,  0, 0 ), (0, 0, 1 , 0), (0, 0, 0, 1))
		down_pattern = ((0, 0, 0 , 0), (0, 1,  0, 0 ), (0, 0, 1 , 0), (0, 0, 0, 1))
		"""
		#http://en.nanotec.com/steppermotor_animation.html
		west_pattern = ((1, 1, 0 , 0), (0, 0, 1 , 1), (1, 1, 0 , 0), (0, 0, 1, 1)) 	# 4 steps to the Left=West / and reset
		east_pattern = ((0, 1, 0, 1), (0, 1, 1, 0), (0, 1, 0, 1), (1, 0, 0, 1)) 		# 4 steps to the Right=East / and reset
		# !Zz; resets all to see that is there
		
		# map irc commands to artbus patterns
		self.irc_patterns = {"west" : west_pattern, "east" : east_pattern}
		
		# artbus command prefixes
		self.command = ('j', 'k') # j = low(0), k = high (1)
		
		
		#maxturn
		self.loopperturn = 4 
		self.maxturns = 6 
		self.turns = {}
		for direction in self.irc_patterns.keys():
			self.turns[direction] = 0
		
		
		# serial config
		if self.TESTF == False:
			self.ser = serial.Serial() # instanciate serial class as ser
			self.ser.port = '/dev/ttyUSB0' #  connection device
			self.ser.baudrate = 115200 # sets the speed
			self.ser.timeout = 0.01 # to not occupy the line forever
			self.single_cmd_break = 0.25	# break between each artbus_command
		else:
			self.single_cmd_break = 0	# break between each artbus_command
		
	
	
	
	# DO COMMAND (!west, !east etc.)
	def DoCommand(self, irc, direction):
		"""takes no arguments

		Translates irc command to artbus command."""
		
		# check irc command: translation to artbus commands possible?
		if self.irc_patterns.has_key(direction) == False:
			e = "unknown direction command. use one of these: " + str(self.irc_patterns.keys())
			irc.error(e)
			return
		
		
		"""
		if self.firstdirection == "":
			self.firstdirection = direction
		irc.reply("firstdir: "+str(self.firstdirection))
		"""
		
		
		# get patterns
		patterns = self.irc_patterns[direction]
		#irc.reply("received direction command: " + direction)
		irc.reply("move cam "+direction+"...")
		
		
		# open serial connection
		if self.TESTF == False:
			try:
				self.ser.open() # opens serial port
			except serial.SerialException, e:
				print e # prints an error if it doesn't work
				irc.error("error: couldn't open serial connection to stepper")
		
		
		# create artbus_commands...
		for i in range(1, self.loopperturn):
			for sequence in patterns: # iterate over `patterns`...
				for bit_number in range(len(sequence)): #...  and use each `tuple`
					bit = sequence[bit_number]
					cmnd = self.command[bit]
					artbus_command = '!S' + str(cmnd) + str(bit_number+1) + ';'
					# print and send artbus_command to serial connection
					print artbus_command
					#irc.reply("send " + artbus_command)
					if self.TESTF == False:
						self.ser.write(artbus_command) # !-bang J-analog board z-reset & semi-end_command
						# read & print response
						response = self.ser.readline() # reads the serial buffer
						print response
						# sleep a while
						time.sleep(self.single_cmd_break)
				print ""	# next sequence of patterns
				
		# close serial connection
		if self.TESTF == False:
			self.ser.close()
		
		
		# reply success
		irc.replySuccess()
	
	
	
	# wrap irc commands
	#for irccommand in self.irc_patterns.keys():
	#	irccommand = wrap(DoCommand)
	
	
	def west(self, irc, msg, args):
		if self.turns["west"] < self.maxturns:
			self.turns["west"] = self.turns["west"] + 1
			#if self.turns["east"] > 0:
			self.turns["east"] = self.turns["east"] - 1
		else:
			e = "maximum numbers of turns exceeded. please turn in the other direction."
			irc.error(e)
			return
		self.DoCommand(irc, "west")	
	west = wrap(west)
	
	
	def east(self, irc, msg, args):
		if self.turns["east"] < self.maxturns:
			self.turns["east"] = self.turns["east"] + 1
			#if self.turns["west"] > 0:
			self.turns["west"] = self.turns["west"] - 1
		else:
			e = "maximum numbers of turns exceeded. please turn in the other direction."
			irc.error(e)
			return
		self.DoCommand(irc, "east")	
	east = wrap(east)
	
	
	"""
	def up(self, irc, msg, args):
		self.DoCommand(irc, "up")	
	up = wrap(up)
	
	def down(self, irc, msg, args):
		self.DoCommand(irc, "down")	
	down = wrap(down)
	"""
	
	
	pass


Class = StepperBot


# vim:set shiftwidth=4 tabstop=4 expandtab textwidth=79:
